@@ -1,11 +1,10 @@
 package Log::Contextual;
+$Log::Contextual::VERSION = '0.006005';
+# ABSTRACT: Simple logging interface with a contextual log
 
 use strict;
 use warnings;
 
-our $VERSION = '0.006000';
-$VERSION = eval $VERSION if $VERSION =~ /_/; # numify for warning-free dev releases
-
 my @levels = qw(debug trace warn info error fatal);
 
 use Exporter::Declare;
@@ -13,10 +12,37 @@ use Exporter::Declare::Export::Generator;
 use Data::Dumper::Concise;
 use Scalar::Util 'blessed';
 
+use B qw(svref_2object);
+
+sub stash_name {
+   my ($coderef) = @_;
+   ref $coderef or return;
+   my $cv = B::svref_2object($coderef);
+   $cv->isa('B::CV') or return;
+
+   # bail out if GV is undefined
+   $cv->GV->isa('B::SPECIAL') and return;
+
+   return $cv->GV->STASH->NAME;
+}
+
 my @dlog = ((map "Dlog_$_", @levels), (map "DlogS_$_", @levels));
 
 my @log = ((map "log_$_", @levels), (map "logS_$_", @levels));
 
+sub _maybe_export {
+   my ($spec, $target, $name, $new_code) = @_;
+
+   if (my $code = $target->can($name)) {
+
+      # this will warn
+      $spec->add_export("&$name", $new_code)
+        unless (stash_name($code) eq __PACKAGE__);
+   } else {
+      $spec->add_export("&$name", $new_code)
+   }
+}
+
 eval {
    require Log::Log4perl;
    die if $Log::Log4perl::VERSION < 1.29;
@@ -27,10 +53,7 @@ eval {
 # export anything but the levels selected
 sub ____ { }
 
-exports ('____',
-   @dlog, @log,
-   qw( set_logger with_logger )
-);
+exports('____', @dlog, @log, qw( set_logger with_logger ));
 
 export_tag dlog => ('____');
 export_tag log  => ('____');
@@ -82,90 +105,83 @@ sub before_import {
       die ref($router) . " does not support set_logger()"
         unless $router->does('Log::Contextual::Role::Router::SetLogger');
 
-      $spec->add_export('&set_logger', sub { $router->set_logger(@_) })
+      _maybe_export($spec, $importer, 'set_logger',
+         sub { $router->set_logger(@_) },
+      );
    }
 
    if ($exports->{'&with_logger'}) {
       die ref($router) . " does not support with_logger()"
         unless $router->does('Log::Contextual::Role::Router::WithLogger');
 
-      $spec->add_export('&with_logger', sub { $router->with_logger(@_) })
+      _maybe_export($spec, $importer, 'with_logger',
+         sub { $router->with_logger(@_) },
+      );
    }
 
    my @levels = @{$class->arg_levels($spec->config->{levels})};
    for my $level (@levels) {
+      my %base =
+        (exporter => $class, caller_level => 1, message_level => $level);
+      my %exports;
       if ($spec->config->{log} || $exports->{"&log_$level"}) {
-         $spec->add_export(
-            "&log_$level",
-            sub (&@) {
-               my ($code, @args) = @_;
-               $router->handle_log_request(
-                  exporter       => $class,
-                  caller_package => scalar(caller),
-                  caller_level   => 1,
-                  message_level  => $level,
-                  message_sub    => $code,
-                  message_args   => \@args,
-               );
-               return @args;
-            });
+         $exports{log_} = sub (&@) {
+            my ($code, @args) = @_;
+            $router->handle_log_request(
+               %base,
+               caller_package => scalar(caller),
+               message_sub    => $code,
+               message_args   => \@args,
+            );
+            return @args;
+         };
       }
       if ($spec->config->{log} || $exports->{"&logS_$level"}) {
-         $spec->add_export(
-            "&logS_$level",
-            sub (&@) {
-               my ($code, @args) = @_;
-               $router->handle_log_request(
-                  exporter       => $class,
-                  caller_package => scalar(caller),
-                  caller_level   => 1,
-                  message_level  => $level,
-                  message_sub    => $code,
-                  message_args   => \@args,
-               );
-               return $args[0];
-            });
+         $exports{logS_} = sub (&@) {
+            my ($code, @args) = @_;
+            $router->handle_log_request(
+               %base,
+               caller_package => scalar(caller),
+               message_sub    => $code,
+               message_args   => \@args,
+            );
+            return $args[0];
+         };
       }
       if ($spec->config->{dlog} || $exports->{"&Dlog_$level"}) {
-         $spec->add_export(
-            "&Dlog_$level",
-            sub (&@) {
-               my ($code, @args) = @_;
-               my $wrapped = sub {
-                  local $_ = (@_ ? Data::Dumper::Concise::Dumper @_ : '()');
-                  &$code;
-               };
-               $router->handle_log_request(
-                  exporter       => $class,
-                  caller_package => scalar(caller),
-                  caller_level   => 1,
-                  message_level  => $level,
-                  message_sub    => $wrapped,
-                  message_args   => \@args,
-               );
-               return @args;
-            });
+         $exports{Dlog_} = sub (&@) {
+            my ($code, @args) = @_;
+            my $wrapped = sub {
+               local $_ = (@_ ? Data::Dumper::Concise::Dumper @_ : '()');
+               &$code;
+            };
+            $router->handle_log_request(
+               %base,
+               caller_package => scalar(caller),
+               message_sub    => $wrapped,
+               message_args   => \@args,
+            );
+            return @args;
+         };
       }
       if ($spec->config->{dlog} || $exports->{"&DlogS_$level"}) {
-         $spec->add_export(
-            "&DlogS_$level",
-            sub (&$) {
-               my ($code, $ref) = @_;
-               my $wrapped = sub {
-                  local $_ = Data::Dumper::Concise::Dumper($_[0]);
-                  &$code;
-               };
-               $router->handle_log_request(
-                  exporter       => $class,
-                  caller_package => scalar(caller),
-                  caller_level   => 1,
-                  message_level  => $level,
-                  message_sub    => $wrapped,
-                  message_args   => [$ref],
-               );
-               return $ref;
-            });
+         $exports{DlogS_} = sub (&$) {
+            my ($code, $ref) = @_;
+            my $wrapped = sub {
+               local $_ = Data::Dumper::Concise::Dumper($_[0]);
+               &$code;
+            };
+            $router->handle_log_request(
+               %base,
+               caller_package => scalar(caller),
+               message_sub    => $wrapped,
+               message_args   => [$ref],
+            );
+            return $ref;
+         };
       }
+      _maybe_export($spec, $importer, $_ . $level, $exports{$_})
+        for keys %exports;
    }
 }
 
@@ -183,20 +199,28 @@ for (qw(set with)) {
    no strict 'refs';
    my $sub = "${_}_logger";
    *{"Log::Contextual::$sub"} = sub {
-      die "$sub is no longer a direct sub in Log::Contextual.  " .
-      'Note that this feature was never tested nor documented.  ' .
-      "Please fix your code to import $sub instead of trying to use it directly"
-   }
+      die "$sub is no longer a direct sub in Log::Contextual.  "
+        . 'Note that this feature was never tested nor documented.  '
+        . "Please fix your code to import $sub instead of trying to use it directly"
+     }
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Log::Contextual - Simple logging interface with a contextual log
 
+=head1 VERSION
+
+version 0.006005
+
 =head1 SYNOPSIS
 
  use Log::Contextual qw( :log :dlog set_logger with_logger );
@@ -276,11 +300,11 @@ C<Log::Contextual> you will be able to swap underlying loggers later.
 
 C<Log::Contextual> chooses which logger to use based on L<< user defined C<CodeRef>s|/LOGGER CODEREF >>.
 Normally you don't need to know this, but you can take advantage of it when you
-need to later
+need to later.
 
 =item * Scalable
 
-If you just want to add logging to your extremely basic application, start with
+If you just want to add logging to your basic application, start with
 L<Log::Contextual::SimpleLogger> and then as your needs grow you can switch to
 L<Log::Dispatchouli> or L<Log::Dispatch> or L<Log::Log4perl> or whatever else.
 
@@ -291,7 +315,7 @@ abstract your logging interface so that logging is as painless as possible,
 while still allowing you to switch from one logger to another.
 
 It is bundled with a really basic logger, L<Log::Contextual::SimpleLogger>,
-but in general you should use a real logger instead of that.  For something
+but in general you should use a real logger instead.  For something
 more serious but not overly complicated, try L<Log::Dispatchouli> (see
 L</SYNOPSIS> for example.)
 
@@ -345,7 +369,7 @@ The C<-package_logger> import option is similar to the C<-logger> import option
 except C<-package_logger> sets the logger for the current package.
 
 Unlike L</-default_logger>, C<-package_logger> cannot be overridden with
-L</set_logger>.
+L</set_logger> or L</with_logger>.
 
  package My::Package;
  use Log::Contextual::SimpleLogger;
@@ -404,7 +428,7 @@ own C<Log::Contextual> subclass as follows:
 
 Note the C<< $_[1] || >> in C<arg_default_logger>.  All of these methods are
 passed the values passed in from the arguments to the subclass, so you can
-either throw them away, honor them, die on usage, or whatever.  To be clear,
+either throw them away, honor them, die on usage, etc.  To be clear,
 if you define your subclass, and someone uses it as follows:
 
  use MyApp::Log::Contextual -default_logger => $foo,
@@ -578,7 +602,7 @@ slurping up (and also setting C<wantarray>) all the C<@args>
 
 Anywhere a logger object can be passed, a coderef is accepted.  This is so
 that the user can use different logger objects based on runtime information.
-The logger coderef is passed the package of the caller the caller level the
+The logger coderef is passed the package of the caller, and the caller level the
 coderef needs to use if it wants more caller information.  The latter is in
 a hashref to allow for more options in the future.
 
@@ -674,14 +698,8 @@ functions. The router singleton is available as the return value of the router()
 of Log::Contextual may overload router() to return instances of custom log routers that
 could for example work with loggers that use a different interface.
 
-=head1 AUTHOR
-
-frew - Arthur Axel "fREW" Schmidt <frioux@gmail.com>
-
 =head1 CONTRIBUTORS
 
-=encoding utf8
-
 triddle - Tyler Riddle <t.riddle@shadowcat.co.uk>
 
 voj - Jakob Voß <voss@gbv.de>
@@ -690,15 +708,15 @@ voj - Jakob Voß <voss@gbv.de>
 
 mst - Matt S. Trout <mst@shadowcat.co.uk>
 
-=head1 COPYRIGHT
+=head1 AUTHOR
 
-Copyright (c) 2012 the Log::Contextual L</AUTHOR> and L</DESIGNER> as listed
-above.
+Arthur Axel "fREW" Schmidt <frioux+cpan@gmail.com>
 
-=head1 LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-This library is free software and may be distributed under the same terms as
-Perl 5 itself.
+This software is copyright (c) 2015 by Arthur Axel "fREW" Schmidt.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
-
